<?php
   class e107HelperCustomField {
      var $id;
      var $fields;
      var $fieldstring;

      function e107HelperCustomField($fieldstring="", $id="") {
         if ($fieldstring == "") {
            $this->fieldstring = "";
            $this->fields = array();
            $this->setId($id);
         } else {
            $this->fieldstring = $fieldstring;
            $this->fields = unserialize(html_entity_decode($fieldstring));
            $this->setId($id);
         }
      }

      function setId($id) {
         if (isset($id) && strlen($id) > 0) {
            // Don't want to set ID to an empty string
            $this->fields["e107HelperCustomField_id"] = $id;
         }
      }

      function addField($id, $field) {
         $this->fields[$id] = $field;
      }

      function count() {
         $count = 0;
         foreach ($this->fields as $key => $value) {
            if (is_array($value)) {
               $count++;
            }
         }
         return $count;
      }

      function getId() {
         return $this->fields["e107HelperCustomField_id"];
      }

      function getLabel($ix, $class="") {
         $ix++; // Increment to bypass id field
         $keys = array_keys($this->fields);
         if (isset($keys[$ix])) {
            $field = $this->fields[$keys[$ix]];
            $pre = "";
            $post = "";
            if ($class != "") {
               // Theme/template supplied
               $pre = "<span class='{$class}'>";
               $post = "</span>";
            } else if (isset($field["label_class"]) && strlen($field["label_class"]) > 0) {
               // Supplied by custom field definition
               $pre = "<span class='".$field["label_class"]."'>";
               $post = "</span>";
            }

            return $pre.$this->fields[$keys[$ix]]["label"].$post;
         }
         return "";
      }

      function getField($ix, $class="") {
         $ix++; // Increment to bypass id field
         $text = "";
         $keys = array_keys($this->fields);
         if (isset($keys[$ix])) {
            $field = $this->fields[$keys[$ix]];
            if (isset($field["field_class"]) && strlen($field["field_class"]) > 0 && $class == "") {
               $class = $field["field_class"];
            }
            switch ($field["fieldtype"]) {
               case "text" : {
                  $text = "<input type='text' class='$class'";
                  if (isset($field["name"]) && strlen($field["name"]) > 0) {
                     $text .= " value='".$this->getPostValue($field["name"])."'";
                     $text .= " name='".$field["name"]."'";
                  }
                  if (isset($field["size"]) && strlen($field["size"]) > 0) {
                     $text .= " size='".$field["size"]."'";
                  }
                  if (isset($field["maxl"]) && strlen($field["maxl"]) > 0) {
                     $text .= " maxlength='".$field["maxl"]."'";
                  }
                  $text .= "/>";
                  break;
               }
               case "textarea" : {
                  $text = "<textarea class='$class'";
                  if (isset($field["name"]) && strlen($field["name"]) > 0) {
                     $text .= " name='".$field["name"]."'";
                  }
                  if (isset($field["rows"]) && strlen($field["rows"]) > 0) {
                     $text .= " rows='".$field["rows"]."'";
                  }
                  if (isset($field["cols"]) && strlen($field["cols"]) > 0) {
                     $text .= " cols='".$field["cols"]."'";
                  }
                  $text .= ">";
                  $text .= $this->getPostValue($field["name"]);
                  $text .= "</textarea>";
                  break;
               }
               case "select" : {
                  $text = "<select class='$class'";
                  if (isset($field["name"]) && strlen($field["name"]) > 0) {
                     $text .= " name='".$field["name"]."'";
                  }
                  if ($field["row1"]) {
                     $text .= "<option value=''>".$field["row1"]."</option>";
                  }
                  $text .= ">";
                  $count = 0;
                  do {
                     $selected = $this->getPostValue($field["name"]) == $field["option{$count}"] ? " selected='true'" : "";
                     $text .= "<option value='".$field["option{$count}"]."'($selected}>".$field["option{$count}"]."</option>";
                     $count++;
                  } while ($this->getValue($field["name"], "option{$count}") != "");
                  $text .= "</select>";
                  break;
               }
               case "date" : {
                  $text = "<input type='text' class='$class'/>";
                  break;
               }
               case "checkbox" : {
                  $checked = $this->getPostValue($field["name"]) == "" ? "" : " checked='true'";
                  $text = "<input type='checkbox' class='$class'$checked";
                  $text .= " value='".$field["value"]."'";
                  if (isset($field["name"]) && strlen($field["name"]) > 0) {
                     $text .= " name='".$field["name"]."'";
                  }
                  $text .= "/>";
                  if (isset($field["text"]) && strlen($field["text"]) > 0) {
                     $text .= $field["text"];
                  }
                  break;
               }
               case "radio" : {
                  $text = "";
                  $count = 0;
                  $name = "";
                  if (isset($field["name"]) && strlen($field["name"]) > 0) {
                     $name = " name='".$field["name"]."'";
                  }
                  do {
                     $checked = $this->getPostValue($field["name"]) == $field["val{$count}"] ? " checked='true'" : "";
                     $text .= "<input type='radio' class='$class'{$name}{$checked} value='".$field["val{$count}"]."'/>".$field["text{$count}"];
                     $count++;
                  } while ($this->getValue($field["name"], "val{$count}") != "");
               }
            }
         }
         return $text;
      }

      function getName($ix) {
         $ix++; // Increment to bypass id field
         $keys = array_keys($this->fields);
         if (isset($keys[$ix])) {
            return $keys[$ix];
         }
         return "";
      }

      function getValue($id, $name) {
         if (isset($this->fields[$id][$name])) {
            return $this->fields[$id][$name];
         }
         return "";
      }

      function getPostValue($name) {
         if (isset($_POST[$name])) {
            return $_POST[$name];
         }
         return "";
      }

      function deleteField($id) {
         if (isset($this->fields[$id])) {
            unset($this->fields[$id]);
            return true;
         } else {
            return false;
         }
      }

      function getAsString($debug=false) {
         if ($debug) {
            $text1 = "";
            $text2 = "";
            foreach ($this->fields as $key => $value) {
               if (is_array($value)) {
                  $expt = var_export($value, true);
                  $expt = str_replace("\n", " ", $expt);
                  $text2 .= "$key = $expt\n";
               } else {
                  $text1 .= "$key = $value\n";
               }
            }
            //$text = str_replace("array", "array\n'", $text);
            return $text1.$text2;
         }
         return serialize($this->fields);
      }

      function getFieldList() {
         $list = "";
         if ($this->count() > 0) {
            $list .= "<table style='display:inline;width:100%;'>";
            foreach ($this->fields as $key => $value) {
               if (is_array($value)) {
                  $list .= "<tr><td>&bull; $key (".$value["fieldtype"].")</td>";
                  $list .= "<td><a href='javascript:e107HelperAjax.addCustomField(\"".e_PLUGIN."e107helpers/e107HelperAjax.php\", \"".$this->getId()."\", 10, \"".$value["fieldtype"]."\", \"".$value["name"]."\")'>";
                  $list .= HELPER_ICON_EDIT."</a></td>";
                  $list .= "<td><a href='javascript:if (jsconfirm(\"".HELPER_LAN_15."?\")) {e107HelperAjax.addCustomField(\"".e_PLUGIN."e107helpers/e107HelperAjax.php\", \"".$this->getId()."\", 20, \"".$value["fieldtype"]."\", \"".$value["name"]."\");}'>";
                  $list .= HELPER_ICON_DELETE."</a></td></tr>"; //<tr><td colspan='3'><hr/></td></tr>";
               }
            }
            $list .= "</table>";
         }
         return $list;
      }


      function getForm($id, $value, $this, $fieldname="") {
         $readonly = ($fieldname=="") ? "" : " readonly='true'";
         $body = "<form>";
         $body .= "<table style='display:inline;'>";
         $body .= "<tr><td>Name</td><td><input id='{$id}_name' name='{$id}_attrib' value='".$this->getValue($fieldname, "name")."' type='text' class='tbox'$readonly/></td></tr>";
         $body .= "<tr><td>Label</td><td><input id='{$id}_label' name='{$id}_attrib' value='".$this->getValue($fieldname, "label")."' type='text' class='tbox'/></td></tr>";
         $body .= "<tr><td>CSS Class for Field</td><td><input id='{$id}_field_class' name='{$id}_attrib' value='".$this->getValue($fieldname, "field_class")."' type='text' class='tbox'/></td></tr>";
         $body .= "<tr><td>CSS Class for Label</td><td><input id='{$id}_label_class' name='{$id}_attrib' value='".$this->getValue($fieldname, "label_class")."' type='text' class='tbox'/></td></tr>";
         switch ($value) {
            case "text" : {
               $header = " : Text";
               $body .= "<tr><td>Size           </td><td><input id='{$id}_size' name='{$id}_attrib' value='".$this->getValue($fieldname, "size")."' type='text' class='tbox'/></td></tr>";
               $body .= "<tr><td>Maximum length </td><td><input id='{$id}_maxl' name='{$id}_attrib' value='".$this->getValue($fieldname, "maxl")."' type='text' class='tbox'/></td></tr>";
               break;
            }
            case "textarea" : {
               $header = " : Text area";
               $body .= "<tr><td>Rows      </td><td><input id='{$id}_rows' name='{$id}_attrib' value='".$this->getValue($fieldname, "rows")."' type='text' class='tbox'/></td></tr>";
               $body .= "<tr><td>Cols/Width</td><td><input id='{$id}_cols' name='{$id}_attrib' value='".$this->getValue($fieldname, "cols")."' type='text' class='tbox'/></td></tr>";
               break;
            }
            case "select" : {
               $header = " : Select (list)";
               $body .= "<tr><td>1st row</td><td><input id='{$id}_row1' name='{$id}_attrib' value='".$this->getValue($fieldname, "row1")."' type='text' class='tbox'/></td></tr>";
               $body .= "<tr><td>Options</td><td>";
               $body .= "<div id='select_container' style='white-space:nowrap;'>";
               $count = 0;
               do {
                  $body .= "<span id='selectline' style='white-space:nowrap;'>";
                  $body .= "<input id='{$id}_option' name='{$id}_option[]' value='".$this->getValue($fieldname, "option{$count}")."' type='text' class='tbox'/>";
                  if ($count > 0) {
                     $body .= "<input type='button' onclick='this.parentNode.parentNode.removeChild(this.parentNode);' class='button' value='x'>";
                     $body .= "<br/></span>";
                  } else {
                     $body .= "</span><br/>";
                  }
                  $count++;
               } while ($this->getValue($fieldname, "option{$count}") != "");
               $body .= "</div></td></tr>";
               $body .= "<tr><td colspan='2' style='text-align:right;'><input type='button' onclick='javascript:duplicateHTML(\"selectline\",\"select_container\");' class='button' value='+'></td></tr>";
               break;
            }
            case "date" : {
               $header = " : Date";
               $body .= "<tr><td>Year from</td><td><input id='{$id}_yearfrom' name='{$id}_attrib' value='".$this->getValue($fieldname, "yearfrom")."' type='text' class='tbox'/></td></tr>";
               $body .= "<tr><td>Year to  </td><td><input id='{$id}_yearto'   name='{$id}_attrib' value='".$this->getValue($fieldname, "yearto")."' type='text' class='tbox'/></td></tr>";
               break;
            }
            case "checkbox" : {
               $header = " : Checkbox";
               $body .= "<tr><td>Value</td><td><input id='{$id}_value' name='{$id}_attrib' value='".$this->getValue($fieldname, "value")."' type='text' class='tbox'/></td></tr>";
               $body .= "<tr><td>Text</td><td><input id='{$id}_text' name='{$id}_attrib' value='".$this->getValue($fieldname, "text")."' type='text' class='tbox'/></td></tr>";
               break;
            }
            case "radio" : {
               $header = " : Radio buttons";
               $body .= "<tr><td>Text/Value</td><td>";
               $body .= "<div id='select_container' style='white-space:nowrap;'>";
               $count = 0;
               do {
                  $body .= "<span id='selectline' style='white-space:nowrap;'>";
                  $body .= "<input id='{$id}_text' name='{$id}_option[]' value='".$this->getValue($fieldname, "text{$count}")."' type='text' class='tbox'/>";
                  $body .= "<input id='{$id}_val'  name='{$id}_option[]' value='".$this->getValue($fieldname, "val{$count}")."' type='text' class='tbox'/>";
                  if ($count > 0) {
                     $body .= "<input type='button' onclick='this.parentNode.parentNode.removeChild(this.parentNode);' class='button' value='x'>";
                     $body .= "<br/></span>";
                  } else {
                     $body .= "</span><br/>";
                  }
                  $count++;
               } while ($this->getValue($fieldname, "val{$count}") != "");
               $body .= "</div></td></tr>";
               $body .= "<tr><td colspan='2' style='text-align:right;'><input type='button' onclick='javascript:duplicateHTML(\"selectline\",\"select_container\");' class='button' value='+'></td></tr>";
               break;
            }
         }
         $body .= "</table>";
         $body .= "</form>";
         return array($header, $body);
      }

      function getDialog($id, $body, $header="", $focus="", $cancel_event="", $cancel_text="Cancel", $ok_event="", $ok_text="Select") {

         // Dialog start
         $text .= "<response type='dialog' id='{$id}_dialog' focus='{$focus}' key='e107CustomFieldKeyHandler'><![CDATA[";
         $text .= "<div style='border:2px outset;padding:1px;'>";
         // Header
         if ($header) {
            $text .= "<div class='forumheader' style='text-align:center;font-size:110%;font-weight:bold;padding:5px;'>$header</div>";
         }
         // Body
         $text .= "<div class='forumheader3' style='padding:15px;'>$body</div>";
         // Help
         $text .= "<div class='forumheader3 smalltext' style='padding:2px;'>Some help text to go in here...</div>";
         // Buttons
         if ($ok_text || $cancel_text) {
            $text .= "<div class='forumheader2' style='text-align:right;padding:5px;'>";
            if ($ok_text) {
               $text .= "<input type='button' class='button' id='{$id}_ok' value='$ok_text' onclick='$ok_event'/>";
            }
            $text .= "&nbsp;";
            if ($cancel_text) {
               $text .= "<input type='button' class='button' value='$cancel_text' onclick='$cancel_event'/>";
            }
            $text .= "<script type='text/javascript'>";
            $text .= "  document.getElementById('{$id}_dialog').onkeypress = function(ev) { e107Helper.dialogKeyHandler(ev, '$cancel_event', '$ok_event');}";
            $text .= "</script>";
            $text .= "</div>";
         }
         // Dialog end
         $text .= "</div>";
         $text .= "]]></response>";
         return $text;
      }
   }
?>